var express = require('express')
var xml = require('xml')
var router = express.Router()

var libs = process.cwd() + '/libs/'

var log = require(libs + 'log')(module)
var config = require(libs + 'config')

var conn = require(libs + 'db/monet')

router.get('/', function (req, res) {
    res.json({
        msg: 'SimCAQ API is running'
    })
})

/**
 * Complete range of the enrollments dataset
 *
 * Returns a tuple of start and ending years of the complete enrollments dataset.
 */
router.get('/year_range', function(req, res) {
    var yearSql = "SELECT MIN(t.ano_censo) AS start_year, MAX(t.ano_censo) AS end_year FROM turmas AS t";
    conn.query(yearSql, true).then(function(result) {
        res.json({
            result: result.data
        })
    });
})

router.get('/data', function(req, res) {
    console.log(req.query)
    console.log(req.query.met)
    console.log(req.query.dim)
    conn.query(
        'SELECT * FROM turmas'
    ).then(function(result) {
        res.json({
            result: result
        })
    })
})

router.get('/enrollments.:format?', function(req, res) {
    var params = req.query;
    var id = 0;
    var location_id = 0;
    var adm_dependency_id = 0;
    var start_year = 0;
    var end_year = 0;
    var enrollmentSql = "";

    if (params.id)
    {
        id = parseInt(params.id, 10);
    }

    if (params.location_id)
    {
        location_id = parseInt(params.location_id, 10);
    }

    if (params.adm_dependency_id)
    {
        adm_dependency_id = parseInt(params.adm_dependency_id)
    }

    if (!params.start_year && !params.end_year)
    {
        var yearSql = "SELECT MIN(t.ano_censo) AS start_year, MAX(t.ano_censo) AS end_year FROM turmas AS t";
        conn.query(yearSql, true).then(function(result) {
            start_year = result.data.start_year;
            end_year = result.data.end_year;
        });
    }

    if (params.start_year)
    {
        start_year = parseInt(params.start_year, 10);
    } else if (start_year > 0) {  // if start_year was not previously set
        var yearSql = "SELECT MIN(t.ano_censo) AS start_year FROM turmas AS t";
        conn.query(yearSql, true).then(function(result) {
            start_year = result.data.start_year;
        });
    }

    if (params.end_year)
    {
        end_year = parseInt(params.end_year, 10);
    } else if (end_year > 0) {  // if end_year was not previously set
        var yearSql = "SELECT MAX(t.ano_censo) AS end_year FROM turmas AS t";
        conn.query(yearSql, true).then(function(result) {
            end_year = result.data.end_year;
        });
    }

    switch(params.aggregate)
    {
    /** TODO: function to compute enrollments by state in the database not yet available
    case "state":
        tbl_name = "matriculas_estado";
        break;
    */
    case "region":
        if (!id) {
            enrollmentSql = "SELECT nome AS name, total, ano_inicio AS start_year, ano_fim AS end_year \
                FROM matRegioes(" + start_year + "," + end_year + "," + adm_dependency_id + "," + location_id + ")";
        } else {
            enrollmentSql = "SELECT nome AS name, total, ano_inicio AS start_year, ano_fim AS end_year \
                FROM matRegiao(" + id + "," + start_year + "," + end_year + "," + adm_dependency_id + "," + location_id + ")";
        }
        break;
    default:
        enrollmentSql = "SELECT nome AS name, total, ano_inicio AS start_year, ano_fim AS end_year \
            FROM matBrasil(" + start_year + "," + end_year + "," + adm_dependency_id + "," + location_id + ")";
    }

    log.debug(params);
    log.debug("Executing query :\"" + enrollmentSql + "\"");

    conn.query(enrollmentSql, true).then(function(result) {
        log.debug(result);
        if (req.params.format === 'csv') {
          res.csv(result.data)
        } else if (req.params.format === 'xml') {
          res.set('Content-Type', 'text/xml')
          res.send(xml({
            result: result.data
          }))
        }
        else {
          res.json({
              result: result.data
          })
        }
    });
})

module.exports = router
